"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var react_dom_1 = require("react-dom");
var mobx_1 = require("mobx");
var mobx_react_1 = require("mobx-react");
var utils_1 = require("./utils");
var modelToObservable = function (app, model) {
    var o = mobx_1.extendObservable({});
    if (model.observable) {
        o = typeof model.observable === 'function' ? model.observable(app) : model.observable;
    }
    // apply state
    if (model.state) {
        mobx_1.extendObservable(o, model.state);
    }
    // apply actions
    if (model.actions) {
        var actions = typeof model.actions === 'function' ? model.actions(app) : model.actions;
        for (var name_1 in actions) {
            mobx_1.extendObservable(o, (_a = {},
                _a[name_1] = mobx_1.action.bound(actions[name_1]),
                _a));
        }
    }
    // apply computed
    if (model.computed) {
        for (var name_2 in model.computed) {
            mobx_1.extendObservable(o, (_b = {},
                _b[name_2] = mobx_1.computed(model.computed[name_2]),
                _b));
        }
    }
    return o;
    var _a, _b;
};
var Cans = (function () {
    function Cans() {
        this.__routerComponent = React.createElement('div');
        this.__models = [];
        this.__modelsObject = {};
    }
    Cans.prototype.__getInjectList = function () {
        return this.__models.map(function (model) { return model.namespace; }).filter(function (_) { return _; });
    };
    Object.defineProperty(Cans.prototype, "models", {
        get: function () {
            return this.__modelsObject;
        },
        enumerable: true,
        configurable: true
    });
    Cans.prototype.route = function (routeFunc) {
        // wrap provider
        var providerProps = {};
        this.__routerComponent = React.createElement(mobx_react_1.Provider, this.models, routeFunc());
    };
    Cans.prototype.model = function (model) {
        // registry model
        var o = modelToObservable(this, model);
        if (utils_1.isProtected(model)) {
            // every model is protected by default
            utils_1.defineReadOnlyProperty(this.__modelsObject, model.namespace, o, "model [" + model.namespace + "] is protected.");
        }
        else {
            this.__modelsObject[model.namespace] = o;
        }
        this.__models.push(model);
    };
    Cans.prototype.start = function (el) {
        if (el === void 0) { el = null; }
        react_dom_1.render(this.__routerComponent, el);
        this.__mountedRoot = el;
    };
    Cans.prototype.use = function (plugin, options) {
        plugin(this, options);
    };
    return Cans;
}());
exports.Cans = Cans;
var createCansApp = function (options) {
    if (options === void 0) { options = {}; }
    if (options.useStrict === true) {
        mobx_1.useStrict(true);
    }
    return new Cans();
};
// model inject helper
exports.inject = function (view) { return mobx_react_1.inject(function (models) { return ({ models: models }); })(mobx_react_1.observer(view)); };
exports.default = createCansApp;
